"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const debug_1 = __importDefault(require("debug"));
const text_stream_accumulator_1 = __importDefault(require("text-stream-accumulator"));
const regex_subscription_1 = __importDefault(require("./searchers/regex-subscription"));
const string_subscription_1 = __importDefault(require("./searchers/string-subscription"));
const debug = debug_1.default("text-stream-search");
class TextStreamSearch {
    constructor(stream) {
        this.subscriptions = [];
        stream.on("data", this.onStreamData.bind(this));
        this.accumulator = new text_stream_accumulator_1.default(stream);
    }
    // Returns the full text received from the stream so far
    fullText() {
        return this.accumulator.toString();
    }
    // Deletes all accumulated text
    reset() {
        this.accumulator.reset();
    }
    // Returns a promise that resolves when the given text shows up in the observed stream.
    // If a timeout iss given, aborts after the given duration.
    async waitForText(text, timeout) {
        debug(`adding text subscription: '${text}'`);
        return new Promise(async (resolve, reject) => {
            this.subscriptions.push(new string_subscription_1.default(text, this.accumulator, resolve, reject, timeout));
            await this.checkSubscriptions();
        });
    }
    // Calls the given handler with the matching text
    // when the given RegExp shows up in the output
    // If a timeout is given, aborts after the given duration.
    waitForRegex(regex, timeout) {
        debug(`adding regex subscription: ${regex.toString()}`);
        return new Promise((resolve, reject) => {
            this.subscriptions.push(new regex_subscription_1.default(regex, this.accumulator, resolve, reject, timeout));
            this.checkSubscriptions();
        });
    }
    // Called when new text arrives
    onStreamData(data) {
        debug(`receiving from stream: '${data}'`);
        // need to wait for the next tick to give the accumulator time to update
        process.nextTick(() => {
            this.checkSubscriptions();
        });
    }
    // Looks for new matches in the received text.
    // Called each time the text or search terms change.
    async checkSubscriptions() {
        debug(`checking ${this.subscriptions.length} subscriptions`);
        const text = this.fullText();
        for (const search of this.subscriptions) {
            await search.check(text);
        }
    }
}
// Add compatibility with `import TextStreamSearch from '...'`
Object.defineProperty(TextStreamSearch, "__esModule", { value: true });
Object.defineProperty(TextStreamSearch, "default", { value: TextStreamSearch });
module.exports = TextStreamSearch;
